from typing_extensions import override
import aiohttp

from .base import GithubService


class GithubServiceImpl(GithubService):
    def __init__(
        self,
        session: aiohttp.ClientSession,
        github_access_token: str,
        base_url: str = "https://api.github.com",
    ):
        self.session: aiohttp.ClientSession = session
        self.base_url: str = base_url
        self.github_access_token: str = github_access_token

    @override
    async def add_pr_comment(
        self,
        github_repo_id: int,
        pull_request_number: int,
        comment_body: str,
    ) -> None:
        """Add a comment to a GitHub pull request.

        Args:
            github_repo_id: Repository ID in format "owner/repo"
            pull_request_number: PR number
            comment_body: Comment text to add
        """
        url = f"{self.base_url}/repos/{github_repo_id}/issues/{pull_request_number}/comments"

        headers = {
            "Authorization": f"Bearer {self.github_access_token}",
            "Accept": "application/vnd.github.v3+json",
            "Content-Type": "application/json",
        }

        data = {"body": comment_body}

        async with self.session.post(url, headers=headers, json=data) as response:
            response.raise_for_status()



from abc import abstractmethod
from typing import Any
from zipfile import ZipFile

from infra_cli.api.types import (
    CreateTaskResponse,
    ExportToGitHubResponse,
    GetTaskResponse,
)


class BaseApiService:
    """Base class for calling the infra.new API backend"""

    def __init__(
        self,
        api_key: str,
        # TODO: update this to point to prod by default
        backend_url: str = "https://infra.new",
    ):
        self.api_key: str = api_key
        self.backend_url: str = backend_url

    @abstractmethod
    async def schedule_drift_task_from_zip(
        self,
        zip_data: str,
        plan_json: dict[Any, Any],  # pyright: ignore[reportExplicitAny]
    ) -> CreateTaskResponse:
        raise NotImplementedError

    @abstractmethod
    async def schedule_drift_task_for_github(
        self,
        github_repo_id: int,
        github_ref: str,
        plan_json: dict[Any, Any],  # pyright: ignore[reportExplicitAny]
    ) -> CreateTaskResponse:
        raise NotImplementedError

    @abstractmethod
    async def poll_background_task(self, task_id: str) -> GetTaskResponse:
        raise NotImplementedError

    @abstractmethod
    async def download_code_result(self, code_result_path: str) -> ZipFile:
        raise NotImplementedError

    @abstractmethod
    async def export_code_to_github(
        self, github_export_path: str
    ) -> ExportToGitHubResponse:
        raise NotImplementedError



from typing import Any
from typing_extensions import override
from infra_cli.terraform.base import BaseTerraformCommandRunner


class FakeTerraformCommandRunner(BaseTerraformCommandRunner):
    @override
    def plan(self, backend_dir: str, refresh_only: bool = True) -> str | None:
        return "plan"

    @override
    def init(self, backend_dir: str) -> None:
        return

    @override
    def show(self, backend_dir: str, plan_path: str) -> dict[str, Any]:  # pyright: ignore[reportExplicitAny]
        return {}



import asyncio
import base64
import datetime
import difflib
import io
import json
import os
from typing import Any
from typing_extensions import override
import zipfile
from pathlib import Path

import typer
from rich import print
from infra_cli.api import BaseApiService
from infra_cli.commands.base import BaseCommand
from infra_cli.github.base import GithubService
from infra_cli.terraform.base import BaseTerraformCommandRunner


class DriftCommand(BaseCommand):
    """Command to check for drift between local and remote state"""

    def __init__(
        self,
        *,
        api: BaseApiService,
        terraform_runner: BaseTerraformCommandRunner,
        github_service: GithubService | None,
        source_dir: str | None,
        backend_dir: str | None,
        plan_file_path: str | None,
        output_dir: str | None,
        github_repo_id: int | None,
        github_ref: str | None,
    ) -> None:
        if not github_repo_id and not source_dir:
            typer.echo("Please provide either a GitHub repo ID or a source directory.")
            raise typer.Exit(1)
        if github_repo_id and source_dir:
            typer.echo("Please provide only one of github_repo_id and source_dir.")
            raise typer.Exit(1)
        if github_repo_id and not github_ref:
            typer.echo("Please provide a GitHub repo ref when providing a GitHub repo.")
            raise typer.Exit(1)
        if github_repo_id and github_ref and not plan_file_path:
            typer.echo("Please provide a plan file path when providing a GitHub repo.")
            raise typer.Exit(1)

        self.api: BaseApiService = api
        self.terraform_runner: BaseTerraformCommandRunner = terraform_runner
        self.github_service: GithubService | None = github_service
        self.source_dir: str | None = source_dir
        self.backend_dir: str | None = (
            os.path.join(source_dir, backend_dir)
            if backend_dir and source_dir
            else source_dir
        )
        self.plan_file_path: str | None = plan_file_path
        self.output_dir: str | None = output_dir
        self.github_repo_id: int | None = github_repo_id
        self.github_ref: str | None = github_ref

    @override
    async def run(self) -> None:
        # First run terraform plan
        if self.plan_file_path is not None:
            with open(self.plan_file_path, "r") as f:
                plan_json = json.loads(f.read())  # pyright: ignore[reportAny]
        elif self.source_dir:
            typer.echo("=== Detecting drift ===")
            plan_path = self._run_terraform_plan()
            if not plan_path:
                typer.echo("No planned changed.")
                return
            plan_json = self._run_terraform_show(plan_path)
        else:
            typer.echo("No source directory provided.")
            return

        typer.echo("\n=== Creating background task ===")
        zip = self._create_zip_from_directory()
        try:
            if zip:
                task = await self.api.schedule_drift_task_from_zip(zip, plan_json)
            elif self.github_repo_id and self.github_ref:
                task = await self.api.schedule_drift_task_for_github(
                    self.github_repo_id, self.github_ref, plan_json
                )
            else:
                raise ValueError("No valid source provided.")

        except Exception as e:
            typer.echo(f"Failed to schedule drift task: {e}")
            raise typer.Exit(1)

        start_time = datetime.datetime.now()

        while True:
            task_response = await self.api.poll_background_task(task.task_id)

            if task_response.status in ["complete", "errored", "terminated"]:
                break
            else:
                typer.echo(
                    f"Drift task is {task_response.status}... [{_format_timedelta(datetime.datetime.now() - start_time)}]"
                )
                await asyncio.sleep(5)

        if task_response.status != "complete":
            typer.echo(
                f"Drift task failed with status: {task_response.status} error: {task_response.error}"
            )
            raise typer.Exit(1)

        if not task_response.code_result_path:
            return

        zip_results: zipfile.ZipFile | None = None

        if self.source_dir:
            zip_results = await self.api.download_code_result(
                task_response.code_result_path
            )

            self._report_diff(zip_results)

        if self.output_dir and zip_results:
            zip_results.extractall(self.output_dir)
        elif task_response.github_export_path:
            typer.echo("\n=== Exporting to GitHub ===")
            export_result = await self.api.export_code_to_github(
                task_response.github_export_path
            )
            typer.echo(f"Created PR for drift fix: {export_result.pr_url}")
            if self.github_service and self.github_repo_id:
                pr_number = int(export_result.pr_url.split("/")[-1])
                await self.github_service.add_pr_comment(
                    self.github_repo_id, pr_number, "Drift fix created successfully"
                )

        if task_response.chat_path:
            typer.echo(
                f"\n\nContinue the chat at: {self.api.backend_url}{task_response.chat_path}"
            )

    def _create_zip_from_directory(self) -> str | None:
        """Create a base64-encoded zip file from a directory."""
        if not self.source_dir:
            return None
        zip_buffer = io.BytesIO()
        with zipfile.ZipFile(zip_buffer, "w", zipfile.ZIP_DEFLATED) as zip_file:
            for root, _, files in os.walk(self.source_dir):
                for file in files:
                    # TODO: we should probably just respect the .gitignore file in the source dir
                    if file.startswith(".terraform.lock"):
                        continue
                    file_path = Path(root, file)

                    if ".terraform" in file_path.parts or ".git" in file_path.parts:
                        continue

                    # Get relative path from source_dir
                    arcname = os.path.relpath(file_path, self.source_dir)
                    zip_file.write(file_path, arcname)

        _ = zip_buffer.seek(0)
        return base64.b64encode(zip_buffer.read()).decode("utf-8")

    def _run_terraform_plan(self) -> str | None:
        if not self.backend_dir:
            return None
        """Run terraform init, plan, and show to get plan JSON."""

        # Initialize terraform
        typer.echo("Running terraform init...")
        self.terraform_runner.init(self.backend_dir)

        typer.echo("Running terraform plan...")
        return self.terraform_runner.plan(self.backend_dir)

    def _run_terraform_show(self, plan_path: str) -> dict[str, Any]:  # pyright: ignore[reportExplicitAny]
        typer.echo("Running terraform show...")
        return self.terraform_runner.show(self.backend_dir or ".", plan_path)

    def _report_diff(self, new_zip: zipfile.ZipFile):
        if not self.source_dir:
            return None
        for file_info in new_zip.filelist:
            with new_zip.open(file_info) as new_file:
                new_file_contents = new_file.read().decode("utf-8")
                old_file_path = os.path.join(self.source_dir, file_info.filename)
                if os.path.exists(old_file_path) and os.path.isfile(old_file_path):
                    with open(old_file_path, "r") as old_file:
                        old_file_contents = old_file.read()

                        diff = difflib.unified_diff(
                            old_file_contents.splitlines(),
                            new_file_contents.splitlines(),
                            fromfile=old_file_path,
                            tofile=file_info.filename,
                        )
                        for d in diff:
                            if d.startswith("---") or d.startswith("+++"):
                                print(f"[yellow]{d}", end="")
                            elif d.startswith("+"):
                                print(f"[green]{d}")
                            elif d.startswith("-"):
                                print(f"[red]{d}")
                            elif d.startswith("@@"):
                                print(f"[bright_magenta]{d}", end="")
                            else:
                                typer.echo(d)
                        if diff:
                            print("\n".join(diff))
                elif os.path.isfile(old_file_path):
                    typer.echo(f"New file: {file_info.filename}")


def _format_timedelta(td: datetime.timedelta) -> str:
    """Pretty print a timedelta object."""
    total_seconds = int(td.total_seconds())
    hours, remainder = divmod(total_seconds, 3600)
    minutes, seconds = divmod(remainder, 60)

    if hours > 0:
        return f"{hours}h {minutes}m {seconds}s"
    elif minutes > 0:
        return f"{minutes}m {seconds}s"
    else:
        return f"{seconds}s"



import json
import os
import subprocess
import tempfile
from typing import Any
from typing_extensions import override
from infra_cli.terraform.base import BaseTerraformCommandRunner


class TerraformCommandRunner(BaseTerraformCommandRunner):
    @override
    def plan(self, backend_dir: str, refresh_only: bool = True) -> str | None:
        temp_dir = tempfile.mkdtemp()
        plan_path = os.path.join(temp_dir, "terraform.tfplan")

        plan_cmd = [
            "terraform",
            "plan",
            "-lock=false",
            "-detailed-exitcode",
            "-out",
            plan_path,
        ]
        if refresh_only:
            plan_cmd.insert(-3, "-refresh-only")

        plan_result = subprocess.run(
            plan_cmd, cwd=backend_dir, capture_output=True, text=True
        )

        if plan_result.returncode == 0:
            return None
        elif plan_result.returncode == 1:
            raise ValueError(f"Terraform plan failed: {plan_result.stderr}")

        return plan_path

    @override
    def init(self, backend_dir: str) -> None:
        # Initialize terraform
        init_result = subprocess.run(
            ["terraform", "init"],
            cwd=backend_dir,
            capture_output=True,
            text=True,
        )
        if init_result.returncode != 0:
            raise RuntimeError(f"Terraform init failed: {init_result.stderr}")

    @override
    def show(self, backend_dir: str, plan_path: str) -> dict[str, Any]:  # pyright: ignore[reportExplicitAny]
        show_result = subprocess.run(
            ["terraform", "show", "-json", plan_path],
            cwd=backend_dir,
            capture_output=True,
            text=True,
        )
        if show_result.returncode != 0:
            raise RuntimeError(f"Terraform show failed: {show_result.stderr}")

        return json.loads(show_result.stdout)  # pyright: ignore[reportAny]



from abc import ABC, abstractmethod


class GithubService(ABC):
    @abstractmethod
    async def add_pr_comment(
        self, github_repo_id: int, pull_request_number: int, comment_body: str
    ) -> None:
        raise NotImplementedError



from abc import abstractmethod
from typing import Any


class BaseTerraformCommandRunner:
    @abstractmethod
    def plan(self, backend_dir: str, refresh_only: bool = True) -> str | None:
        raise NotImplementedError

    @abstractmethod
    def init(self, backend_dir: str) -> None:
        raise NotImplementedError

    @abstractmethod
    def show(self, backend_dir: str, plan_path: str) -> dict[str, Any]:
        raise NotImplementedError



import asyncio
import os
import shutil
import tempfile
from zipfile import ZipFile
from typing_extensions import override
import unittest

import pytest

from infra_cli.api.testing.fake_api import FakeApiService
from infra_cli.api.types import (
    CreateTaskResponse,
    ExportToGitHubResponse,
    GetTaskResponse,
)
from infra_cli.commands.drift import DriftCommand
from infra_cli.terraform.testing.fake_command_runner import FakeTerraformCommandRunner


class TestDriftCommand(unittest.TestCase):
    @override
    def setUp(self) -> None:
        self.fake_api: FakeApiService = FakeApiService()  # pyright: ignore[reportUninitializedInstanceVariable]
        self.fake_tf_runner: FakeTerraformCommandRunner = FakeTerraformCommandRunner()  # pyright: ignore[reportUninitializedInstanceVariable]
        self.source_dir: str = tempfile.mkdtemp()  # pyright: ignore[reportUninitializedInstanceVariable]
        self.backend_dir: str = os.path.join(self.source_dir, "backend")  # pyright: ignore[reportUninitializedInstanceVariable]
        os.mkdir(self.backend_dir)
        with open(os.path.join(self.backend_dir, "temp.tf"), "w") as f:
            _ = f.write("# this is a comment")

        self.base_new_zip_dir: str = tempfile.mkdtemp()  # pyright: ignore[reportUninitializedInstanceVariable]
        new_zip_backend_dir = os.path.join(self.base_new_zip_dir, "backend")
        new_zip_file: str = os.path.join(self.base_new_zip_dir, "backend", "temp.tf")
        os.mkdir(new_zip_backend_dir)
        with open(os.path.join(new_zip_backend_dir, "temp.tf"), "w") as f:
            _ = f.write("# this is a new comment")

        self.new_zip_file: tuple[int, str] = tempfile.mkstemp(suffix=".zip")  # pyright: ignore[reportUninitializedInstanceVariable]

        with open(self.new_zip_file[1], "wb") as f:
            with ZipFile(f, "w") as zip_file:
                zip_file.write(new_zip_file, arcname="backend/temp.tf")

    @pytest.fixture(autouse=True)
    def capsys(self, capsys: pytest.CaptureFixture[str]) -> None:  # pyright: ignore[reportUninitializedInstanceVariable,reportRedeclaration]
        self.capsys: pytest.CaptureFixture[str] = capsys

    @override
    def tearDown(self) -> None:
        shutil.rmtree(self.source_dir)
        shutil.rmtree(self.base_new_zip_dir)
        os.remove(self.new_zip_file[1])

    def test_with_provided_plan_file(self):
        plan_file = os.path.join(self.base_new_zip_dir, "plan.json")
        with open(plan_file, "w") as f:
            _ = f.write("{}")

        self.fake_api.schedule_drift_task_from_zip_result = [
            CreateTaskResponse(task_id="task-id")
        ]
        self.fake_api.poll_background_task_result = [
            GetTaskResponse(status="complete", code_result_path="/code/path")
        ]

        with ZipFile(self.new_zip_file[1]) as f:
            self.fake_api.download_code_result_result = [f]

            drift = DriftCommand(
                api=self.fake_api,
                terraform_runner=self.fake_tf_runner,
                source_dir=self.source_dir,
                backend_dir=self.backend_dir,
                plan_file_path=plan_file,
                output_dir=None,
                github_ref=None,
                github_repo_id=None,
            )
            asyncio.run(drift.run())

        self.assertEqual(self.fake_api.schedule_drift_task_from_zip_call_count, 1)
        self.assertEqual(self.fake_api.poll_background_task_call_count, 1)
        self.assertEqual(self.fake_api.download_code_result_call_count, 1)

        out, _ = self.capsys.readouterr()

        self.assertIn("-# this is a comment", out)
        self.assertIn("+# this is a new comment", out)

    def test_without_provided_plan_file(self):
        self.fake_api.schedule_drift_task_from_zip_result = [
            CreateTaskResponse(task_id="task-id")
        ]
        self.fake_api.poll_background_task_result = [
            GetTaskResponse(status="complete", code_result_path="/code/path")
        ]

        with ZipFile(self.new_zip_file[1]) as f:
            self.fake_api.download_code_result_result = [f]

            drift = DriftCommand(
                api=self.fake_api,
                terraform_runner=self.fake_tf_runner,
                source_dir=self.source_dir,
                backend_dir=self.backend_dir,
                plan_file_path=None,
                output_dir=None,
                github_ref=None,
                github_repo_id=None,
            )
            asyncio.run(drift.run())

        self.assertEqual(self.fake_api.schedule_drift_task_from_zip_call_count, 1)
        self.assertEqual(self.fake_api.poll_background_task_call_count, 1)
        self.assertEqual(self.fake_api.download_code_result_call_count, 1)

    def test_with_output_dir(self):
        self.fake_api.schedule_drift_task_from_zip_result = [
            CreateTaskResponse(task_id="task-id")
        ]
        self.fake_api.poll_background_task_result = [
            GetTaskResponse(status="complete", code_result_path="/code/path")
        ]

        with ZipFile(self.new_zip_file[1]) as f:
            with tempfile.TemporaryDirectory() as output_dir:
                self.fake_api.download_code_result_result = [f]

                drift = DriftCommand(
                    api=self.fake_api,
                    terraform_runner=self.fake_tf_runner,
                    source_dir=self.source_dir,
                    backend_dir=self.backend_dir,
                    plan_file_path=None,
                    output_dir=output_dir,
                    github_ref=None,
                    github_repo_id=None,
                )
                asyncio.run(drift.run())

                with open(os.path.join(output_dir, "backend", "temp.tf")) as f:
                    contents = f.read()
                    self.assertEqual(contents, "# this is a new comment")

    def test_with_git(self):
        plan_file = os.path.join(self.base_new_zip_dir, "plan.json")
        with open(plan_file, "w") as f:
            _ = f.write("{}")
        self.fake_api.schedule_drift_task_for_github_result = [
            CreateTaskResponse(task_id="task-id")
        ]
        self.fake_api.poll_background_task_result = [
            GetTaskResponse(
                status="complete",
                code_result_path="/code/path",
                github_export_path="/github/export",
            )
        ]
        self.fake_api.export_code_to_github_result = [
            ExportToGitHubResponse(pr_url="url")
        ]

        drift = DriftCommand(
            api=self.fake_api,
            terraform_runner=self.fake_tf_runner,
            source_dir=None,
            backend_dir=None,
            plan_file_path=plan_file,
            output_dir=None,
            github_ref="main",
            github_repo_id=12345,
        )
        asyncio.run(drift.run())

        self.assertEqual(self.fake_api.schedule_drift_task_for_github_call_count, 1)
        self.assertEqual(self.fake_api.schedule_drift_task_from_zip_call_count, 0)
        self.assertEqual(self.fake_api.export_code_to_github_call_count, 1)



from functools import partial
from typing import Annotated
import aiohttp
import typer

from infra_cli.api import ApiService
from infra_cli.commands.drift import DriftCommand
from asyncer import syncify
from dotenv import load_dotenv

from infra_cli.github.service import GithubServiceImpl
from infra_cli.terraform.command_runner import TerraformCommandRunner

_ = load_dotenv()

app = typer.Typer(pretty_exceptions_enable=False)


@app.command()
@partial(syncify, raise_sync_error=False)
async def drift(
    api_key: Annotated[str, typer.Option(help="API key")],
    github_repo_id: Annotated[int | None, typer.Option(help="GitHub repo ID")] = None,
    github_ref: Annotated[str | None, typer.Option(help="GitHub repo ref")] = None,
    source_dir: Annotated[
        str | None,
        typer.Option(help="This is the source directory that will be uploaded."),
    ] = None,
    backend_dir: Annotated[
        str | None,
        typer.Option(
            help="The directory where the backend is located. This should be a relative path from the source directory. This is only needed if you are not passing in a terraform plan file."
        ),
    ] = None,
    output_dir: Annotated[str | None, typer.Option(help="Output directory")] = None,
    plan_file_path: Annotated[
        str | None, typer.Option(help="Path to plan file")
    ] = None,
    # Hidden arguments / options
    backend_url: Annotated[
        str, typer.Option(help="Backend URL", hidden=True)
    ] = "https://infra.new",
    github_access_token: Annotated[
        str | None, typer.Option(help="Github access token")
    ] = None,
):
    tf_cmd_runner = TerraformCommandRunner()
    async with aiohttp.ClientSession() as session:
        api = ApiService(session, api_key, backend_url)
        gh_service = (
            GithubServiceImpl(session, github_access_token)
            if github_access_token
            else None
        )
        cmd = DriftCommand(
            api=api,
            terraform_runner=tf_cmd_runner,
            source_dir=source_dir,
            backend_dir=backend_dir,
            plan_file_path=plan_file_path,
            output_dir=output_dir,
            github_repo_id=github_repo_id,
            github_ref=github_ref,
            github_service=gh_service,
        )

        await cmd.run()


@app.command()
def version():
    typer.echo("Version action")


if __name__ == "__main__":
    app()



from dataclasses import dataclass
from typing import Any


@dataclass
class CreateTaskRequest:
    plan_json: dict[Any, Any]  # pyright: ignore[reportExplicitAny]
    zip: str | None = None
    github_repo_id: int | None = None
    github_ref: str | None = None
    additionalContext: str | None = None

    def to_json(self):
        return {
            "zip": self.zip,
            "planJson": self.plan_json,
            "additionalContext": self.additionalContext,
            "githubRepoId": self.github_repo_id,
            "githubRef": self.github_ref,
        }


@dataclass
class CreateTaskResponse:
    task_id: str

    @classmethod
    def from_json(cls, json: dict[str, str]) -> "CreateTaskResponse":
        return cls(task_id=json["taskId"])


@dataclass
class GetTaskResponse:
    status: str
    code_result_path: str | None = None
    github_export_path: str | None = None
    error: str | None = None
    chat_path: str | None = None

    @classmethod
    def from_json(cls, json: dict[str, str]) -> "GetTaskResponse":
        return cls(
            status=json["status"],
            code_result_path=json.get("codeResultPath"),
            github_export_path=json.get("githubExportPath"),
            error=json.get("error"),
            chat_path=json.get("chatPath"),
        )


@dataclass
class ExportToGitHubResponse:
    pr_url: str

    @classmethod
    def from_json(cls, json: dict[str, str]) -> "ExportToGitHubResponse":
        return cls(pr_url=json["prUrl"])



from typing import Any
from zipfile import ZipFile
from typing_extensions import override
from infra_cli.api.base import BaseApiService
from infra_cli.api.types import (
    CreateTaskResponse,
    ExportToGitHubResponse,
    GetTaskResponse,
)


class FakeApiService(BaseApiService):
    def __init__(
        self,
        api_key: str = "unused",
        backend_url: str = "https://infra.new",
    ):
        super().__init__(api_key, backend_url)

        self.schedule_drift_task_from_zip_result: (
            list[CreateTaskResponse] | Exception | None
        ) = None
        self.schedule_drift_task_for_github_result: (
            list[CreateTaskResponse] | Exception | None
        ) = None
        self.poll_background_task_result: list[GetTaskResponse] | Exception | None = (
            None
        )
        self.download_code_result_result: list[ZipFile] | Exception | None = None
        self.export_code_to_github_result: (
            list[ExportToGitHubResponse] | ExceptionGroup | None
        ) = None

        self.schedule_drift_task_from_zip_call_count: int = 0
        self.schedule_drift_task_for_github_call_count: int = 0
        self.poll_background_task_call_count: int = 0
        self.download_code_result_call_count: int = 0
        self.export_code_to_github_call_count: int = 0

    @override
    async def schedule_drift_task_from_zip(
        self,
        zip_data: str,
        plan_json: dict[Any, Any],  # pyright: ignore[reportExplicitAny]
    ) -> CreateTaskResponse:
        if self.schedule_drift_task_from_zip_result is None:
            raise ValueError(
                "No response for schedule_drift_task_from_zip was provided"
            )

        self.schedule_drift_task_from_zip_call_count += 1

        if isinstance(self.schedule_drift_task_from_zip_result, Exception):
            raise self.schedule_drift_task_from_zip_result

        if len(self.schedule_drift_task_from_zip_result) == 0:
            raise ValueError("schedule_drift_task_from_zip is empty")
        return self.schedule_drift_task_from_zip_result.pop(0)

    @override
    async def schedule_drift_task_for_github(
        self,
        github_repo_id: int,
        github_ref: str,
        plan_json: dict[Any, Any],  # pyright: ignore[reportExplicitAny]
    ) -> CreateTaskResponse:
        if self.schedule_drift_task_for_github_result is None:
            raise ValueError(
                "No response for schedule_drift_task_for_github was provided"
            )

        self.schedule_drift_task_for_github_call_count += 1

        if isinstance(self.schedule_drift_task_for_github_result, Exception):
            raise self.schedule_drift_task_for_github_result

        if len(self.schedule_drift_task_for_github_result) == 0:
            raise ValueError("schedule_drift_task_for_github_result is empty")
        return self.schedule_drift_task_for_github_result.pop(0)

    @override
    async def poll_background_task(self, task_id: str) -> GetTaskResponse:
        if self.poll_background_task_result is None:
            raise ValueError("No response for poll_background_task was provided")

        self.poll_background_task_call_count += 1

        if isinstance(self.poll_background_task_result, Exception):
            raise self.poll_background_task_result

        if len(self.poll_background_task_result) == 0:
            raise ValueError("poll_background_task_result is empty")

        return self.poll_background_task_result.pop(0)

    @override
    async def download_code_result(self, code_result_path: str) -> ZipFile:
        if self.download_code_result_result is None:
            raise ValueError("No response for download_code_result_result was provided")

        self.download_code_result_call_count += 1

        if isinstance(self.download_code_result_result, Exception):
            raise self.download_code_result_result

        if len(self.download_code_result_result) == 0:
            raise ValueError("download_code_result_result is empty")

        return self.download_code_result_result.pop(0)

    @override
    async def export_code_to_github(
        self, github_export_path: str
    ) -> ExportToGitHubResponse:
        if self.export_code_to_github_result is None:
            raise ValueError(
                "No response for export_code_to_github_result was provided"
            )

        self.export_code_to_github_call_count += 1

        if isinstance(self.export_code_to_github_result, Exception):
            raise self.export_code_to_github_result

        if len(self.export_code_to_github_result) == 0:
            raise ValueError("export_code_to_github_result is empty")

        return self.export_code_to_github_result.pop(0)



import io
import os
from typing import Any
from zipfile import ZipFile
import aiohttp
from typing_extensions import override

from infra_cli.api.types import (
    CreateTaskRequest,
    CreateTaskResponse,
    ExportToGitHubResponse,
    GetTaskResponse,
)
from .base import BaseApiService


class ApiService(BaseApiService):
    """Service for calling the infra.new API backend"""

    def __init__(
        self,
        session: aiohttp.ClientSession,
        api_key: str,
        # TODO: update this to point to prod by default
        backend_url: str = "https://infra.new",
    ):
        super().__init__(api_key, backend_url)
        self.session: aiohttp.ClientSession = session
        self._cf_access_client_id: str | None = os.environ.get("CF_ACCESS_CLIENT_ID")
        self._cf_access_client_secret: str | None = os.environ.get(
            "CF_ACCESS_CLIENT_SECRET"
        )
        self._base_headers: dict[str, str] = {
            "Content-Type": "application/json",
            "Authorization": self.api_key,
        }
        if self._cf_access_client_id:
            self._base_headers["CF-Access-Client-Id"] = self._cf_access_client_id
        if self._cf_access_client_secret:
            self._base_headers["CF-Access-Client-Secret"] = (
                self._cf_access_client_secret
            )

    @override
    async def schedule_drift_task_from_zip(
        self,
        zip_data: str,
        plan_json: dict[Any, Any],  # pyright: ignore[reportExplicitAny]
    ) -> CreateTaskResponse:
        """Schedule a drift task for the given plan path"""

        request = CreateTaskRequest(zip=zip_data, plan_json=plan_json)

        response = await self.session.post(
            f"{self.backend_url}/api/experimental/tasks",
            headers=self._base_headers,
            json=request.to_json(),
        )

        response.raise_for_status()

        json_response: dict[str, str] = await response.json(content_type="text/plain")

        return CreateTaskResponse.from_json(json_response)

    @override
    async def schedule_drift_task_for_github(
        self,
        github_repo_id: int,
        github_ref: str,
        plan_json: dict[Any, Any],  # pyright: ignore[reportExplicitAny]
    ) -> CreateTaskResponse:
        """Schedule a drift task for the given plan path"""

        request = CreateTaskRequest(
            github_repo_id=github_repo_id,
            github_ref=github_ref,
            plan_json=plan_json,
        )

        response = await self.session.post(
            f"{self.backend_url}/api/experimental/tasks",
            headers=self._base_headers,
            json=request.to_json(),
        )

        if response.status not in [200, 201]:
            error_response = await response.text()
            raise ValueError(error_response)

        json_response: dict[str, str] = await response.json(content_type="text/plain")

        return CreateTaskResponse.from_json(json_response)

    @override
    async def poll_background_task(self, task_id: str) -> GetTaskResponse:
        response = await self.session.get(
            f"{self.backend_url}/api/experimental/tasks/{task_id}",
            headers=self._base_headers,
        )

        response.raise_for_status()

        json_response: dict[str, str] = await response.json()

        return GetTaskResponse.from_json(json_response)

    @override
    async def download_code_result(self, code_result_path: str) -> ZipFile:
        response = await self.session.get(
            f"{self.backend_url}{code_result_path}",
            headers=self._base_headers,
        )

        response.raise_for_status()

        zip_bytes = await response.read()
        return ZipFile(io.BytesIO(zip_bytes))

    @override
    async def export_code_to_github(
        self, github_export_path: str
    ) -> ExportToGitHubResponse:
        response = await self.session.post(
            f"{self.backend_url}{github_export_path}",
            headers=self._base_headers,
            json={},
        )

        response.raise_for_status()

        json_response: dict[str, str] = await response.json()

        return ExportToGitHubResponse.from_json(json_response)
